@@ -10,32 +10,37 @@ DO_CLEAN=${DO_CLEAN:-1}
 DO_CUDA=${DO_CUDA:-1}
 DO_ZIP=${DO_ZIP:-0}
 DO_INSTALLER=${DO_INSTALLER:-1}
-DO_TESTS=${DO_TESTS:-1}
 DO_VERPATCH=${DO_VERPATCH:-1}
-DO_SERVICE=${DO_SERVICE:-1}
 DO_DOC=${DO_DOC:-1}
 CLIENT_ONLY=${CLIENT_ONLY:-0}
 RUN_INSTALLER=${RUN_INSTALLER:-1}
 DO_MSI=${DO_MSI:-0}
 DO_SIGN=${DO_SIGN:-1}
 BUNDLE_PUTTY=${BUNDLE_PUTTY:-1}
-BUNDLE_OPENSSH=${BUNDLE_OPENSSH:-0}
-BUNDLE_OPENSSL=${BUNDLE_OPENSSL:-1}
-BUNDLE_PAEXEC=${BUNDLE_PAEXEC:-1}
-BUNDLE_DESKTOPLOGON=${BUNDLE_DESKTOPLOGON:-1}
+if [ "${CLIENT_ONLY}" == "1" ]; then
+	BUNDLE_OPENSSH=${BUNDLE_OPENSSH:-0}
+	BUNDLE_OPENSSL=${BUNDLE_OPENSSL:-0}
+	BUNDLE_PAEXEC=${BUNDLE_PAEXEC:-0}
+	BUNDLE_DESKTOPLOGON=${BUNDLE_DESKTOPLOGON:-0}
+	DO_TESTS=${DO_TESTS:-0}
+	DO_SERVICE=${DO_SERVICE:-0}
+else
+	BUNDLE_OPENSSH=${BUNDLE_OPENSSH:-1}
+	BUNDLE_OPENSSL=${BUNDLE_OPENSSL:-1}
+	BUNDLE_PAEXEC=${BUNDLE_PAEXEC:-1}
+	BUNDLE_DESKTOPLOGON=${BUNDLE_DESKTOPLOGON:-1}
+	DO_TESTS=${DO_TESTS:-1}
+	DO_SERVICE=${DO_SERVICE:-1}
+fi
 ZIP_MODULES=${ZIP_MODULES:-1}
 
 PYTHON=python3
 
 KEY_FILE="E:\\xpra.pfx"
 DIST="./dist"
-BUILD_OPTIONS="--without-enc_x265 --without-cuda_rebuild"
+BUILD_OPTIONS=""
 
 if [ "${CLIENT_ONLY}" == "1" ]; then
-	DO_TESTS="0"
-	DO_SERVICE="0"
-	BUNDLE_PAEXEC="0"
-	BUNDLE_DESKTOPLOGON="0"
 	BUILD_OPTIONS="${BUILD_OPTIONS} --without-enc_x264 --without-nvenc --without-nvfbc"
 	shift
 fi
@@ -161,6 +166,7 @@ if [ "${DO_CLEAN}" == "1" ]; then
 	#clean sometimes errors on removing pyd files,
 	#so do it with rm instead:
 	find xpra/ -name "*-cpython-*dll" -exec rm -f {} \;
+	find xpra/ -name "*.cp*-mingw*.pyd" -exec rm -f {} \;
 	CLEAN_LOG="clean.log"
 	${PYTHON} ./setup.py clean >& "${CLEAN_LOG}"
 	if [ "$?" != "0" ]; then
@@ -259,17 +265,19 @@ fi
 
 if [ "${DO_CUDA}" == "1" ]; then
 	echo "* Building CUDA kernels"
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRA_to_NV12 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRA_to_YUV444 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" ARGB_to_NV12 || exit 1
-	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" ARGB_to_YUV444 || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_NV12 || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_YUV444 || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" XRGB_to_NV12 || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" XRGB_to_YUV444 || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" BGRX_to_RGB || exit 1
+	cmd.exe //c "packaging\\MSWindows\\BUILD_CUDA_KERNEL" RGBX_to_RGB || exit 1
 else
 	BUILD_OPTIONS="${BUILD_OPTIONS} --without-nvenc"
 fi
 
 echo "* Building Python3 Cython modules"
 BUILD_LOG="packaging/MSWindows/Python3-build.log"
-${PYTHON} ./setup.py build_ext ${BUILD_OPTIONS} --inplace >& ${BUILD_LOG}
+${PYTHON} ./setup.py build_ext ${BUILD_OPTIONS} --inplace -j `nproc` >& ${BUILD_LOG}
 if [ "$?" != "0" ]; then
 	echo "ERROR: build failed, see ${BUILD_LOG}:"
 	tail -n 20 "${BUILD_LOG}"
@@ -348,6 +356,8 @@ mv ./libgst*.dll ./lib/gstreamer-1.0/
 mv ./lib/gstreamer-1.0/libgstreamer*.dll ./lib/
 #and the gstreamer support libraries look like plugins but those are actual DLLs:
 mv ./lib/gstreamer-1.0/libgst*-1.0-*.dll ./lib/gstreamer-1.0/libwavpack* ./lib/
+#gstreamer modules don't have "-" in the DLL name:
+mv ./lib/gstreamer-1.0/lib*-?.dll ./lib/
 #move most DLLs to /lib
 mv *dll lib/
 #but keep the core DLLs (python, gcc, etc):
@@ -360,8 +370,28 @@ for x in `ls *dll`; do
 	find ./ -mindepth 2 -name "${x}" -exec rm {} \;
 done
 
+#remove PIL loaders we don't use:
+echo "* removing unnecessary PIL plugins:"
+pushd "./PIL"
+RMP=""
+KMP=""
+for file_name in `ls *Image*`; do
+	plugin_name=`echo $file_name | sed 's+\.py.*++g'`
+	echo "$file_name" | egrep "Bmp|Ico|Image.py|ImageChops|ImageCms|ImageWin|ImageChops|ImageColor|ImageDraw|ImageFile|ImageFilter|ImageFont|ImageGrab|ImageMode|ImageOps|ImagePalette|ImagePath|ImageSequence|ImageStat|ImageTransform|Jpeg|Tiff|Png|Ppm|Xpm|WebP" >& /dev/null
+	if [ "$?" == "0" ]; then
+		KMP="${KMP} $plugin_name"
+	else
+		RMP="${RMP} $plugin_name"
+		rm $file_name
+	fi
+done
+echo " removed: ${RMP}"
+echo " kept: ${KMP}"
+popd
+
 #remove test bits we don't need:
 rm -fr ./future/backports/test ./comtypes/test/ ./ctypes/macholib/fetch_macholib* ./distutils/tests ./distutils/command ./enum/doc ./websocket/tests ./email/test/
+rm -fr ./Crypto/SelfTest/*
 #remove source:
 find xpra -name "*.pyx" -exec rm {} \;
 find xpra -name "*.c" -exec rm {} \;
@@ -387,7 +417,14 @@ if [ "${ZIP_MODULES}" == "1" ]; then
 			http enum sqlite3 winreg copyreg _thread _dummythread builtins importlib \
 			logging queue urllib xml xmlrpc pyasn1_modules concurrent pynvml collections > /dev/null
 fi
+#leave ./lib
 popd > /dev/null
+#remove empty icon directories
+for i in `seq 4`; do
+	find share/icons -type d -exec rmdir {} \; 2> /dev/null
+done
+rm -fr share/xml
+#leave ./dist
 popd > /dev/null
 
 
@@ -460,9 +497,9 @@ if [ "${DO_VERPATCH}" == "1" ]; then
 		tool_name=`echo $exe | sed 's+dist/++g;s+Xpra_++g;s+Xpra-++g;s+_+ +g;s+-+ +g;s+\.exe++g'`
 		verpatch $exe				//s desc "Xpra $tool_name"		//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	done
-	verpatch dist/Xpra_cmd.exe 		//s desc "Xpra command line"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra_cmd.exe		//s desc "Xpra command line"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	verpatch dist/Xpra-Proxy.exe	//s desc "Xpra Proxy Server"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
-	verpatch dist/Xpra.exe 			//s desc "Xpra" 				//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra.exe			//s desc "Xpra"					//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 fi
 
 ################################################################################
