@@ -1,5 +1,5 @@
-#!/usr/bin/env python
-# Copyright (C) 2018-2020 Antoine Martin <antoine@xpra.org>
+#!/usr/bin/env python3
+# Copyright (C) 2018-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -45,7 +45,6 @@ class FakeApplication:
         self.server_clipboard = False
         self.server_bandwidth_limit_change = 0
         self.server_encodings = ["png", "rgb"]
-        self.server_encodings_problematic = []
         self.server_encodings_with_quality = []
         self.server_encodings_with_speed = []
         self.server_start_new_commands = True
@@ -175,7 +174,7 @@ class FakeApplication:
         log("xpra_tray_geometry%s geometry=%s", args, self.tray.get_geometry())
 
 
-    def disconnect_and_quit(self, *args):
+    def disconnect_and_quit(self, *_args):
         Gtk.main_quit()
 
 
