@@ -5,6 +5,7 @@
 # later version. See the file COPYING for details.
 
 import os
+from time import monotonic
 from collections import namedtuple
 from ctypes import (
     create_string_buffer, create_unicode_buffer,
@@ -12,11 +13,11 @@ from ctypes import (
     )
 from ctypes.wintypes import RECT, POINT, BYTE, MAX_PATH
 
-from xpra.os_util import strtobytes, monotonic_time
+from xpra.os_util import strtobytes
 from xpra.util import envbool, prettify_plug_name, csv, XPRA_APP_ID, XPRA_IDLE_NOTIFICATION_ID
 from xpra.scripts.config import InitException
 from xpra.server.gtk_server_base import GTKServerBase
-from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase, MULTI_WINDOW
+from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
 from xpra.server.shadow.root_window_model import RootWindowModel
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.gui import get_desktop_name, get_fixed_cursor_size
@@ -341,7 +342,7 @@ class SeamlessRootWindowModel(RootWindowModel):
 class ShadowServer(GTKShadowServerBase):
 
     def __init__(self):
-        GTKShadowServerBase.__init__(self)
+        super().__init__()
         self.keycodes = {}
         self.cursor_handle = None
         self.cursor_data = None
@@ -391,7 +392,7 @@ class ShadowServer(GTKShadowServerBase):
         w, h = self.get_root_window_size()
         try:
             display = prettify_plug_name(self.root.get_screen().get_display().get_name())
-        except:
+        except Exception:
             display = ""
         self.do_print_screen_info(display, w, h)
 
@@ -406,33 +407,28 @@ class ShadowServer(GTKShadowServerBase):
         w, h = get_root_window_size()
         return init_capture(w, h, self.pixel_depth)
 
-    def makeRootWindowModels(self):
-        log("makeRootWindowModels() root=%s", self.root)
-        self.capture = self.setup_capture()
+
+    def get_root_window_model_class(self):
         if SEAMLESS:
-            model_class = SeamlessRootWindowModel
-        else:
-            model_class = RootWindowModel
-        if not MULTI_WINDOW:
-            return (model_class(self.root, self.capture),)
-        models = []
-        monitors = get_monitors()
-        for i, monitor in enumerate(monitors):
+            return SeamlessRootWindowModel
+        return RootWindowModel
+
+    def get_shadow_monitors(self):
+        #convert to the format expected by GTKShadowServerBase:
+        monitors = []
+        for i, monitor in enumerate(get_monitors()):
             geom = monitor["Monitor"]
             x1, y1, x2, y2 = geom
             assert x1<x2 and y1<y2
-            model = model_class(self.root, self.capture)
-            model.title = monitor["Device"].lstrip("\\\\.\\")
-            model.geometry = x1, y1, x2-x1, y2-y1
-            screenlog("monitor %i: %10s geometry=%s (from %s)", i, model.title, model.geometry, geom)
-            models.append(model)
-            screenlog("makeRootWindowModels: model(%s)=%s", monitor, model)
-        log("makeRootWindowModels()=%s", models)
-        return models
+            plug_name = monitor["Device"].lstrip("\\\\.\\")
+            monitors.append((plug_name, x1, y1, x2-x1, y2-y1, 1))
+            screenlog("monitor %i: %10s coordinates: %s", i, plug_name, geom)
+        log("get_shadow_monitors()=%s", monitors)
+        return monitors
 
 
     def refresh(self):
-        v = GTKShadowServerBase.refresh(self)
+        v = super().refresh()
         if v and SEAMLESS:
             for rwm in self._id_to_window.values():
                 rwm.refresh_shape()
@@ -470,26 +466,29 @@ class ShadowServer(GTKShadowServerBase):
         return pos.x, pos.y
 
     def do_process_mouse_common(self, proto, wid, pointer, *_args):
+        ss = self._server_sources.get(proto)
+        if not ss:
+            return False
         #adjust pointer position for offset in client:
         try:
             x, y = pointer[:2]
-            if not SetPhysicalCursorPos(x, y):
-                #rate limit the warnings:
-                start, count = self.cursor_errors
-                now = monotonic_time()
-                elapsed = now-start
-                if count==0 or (count>1 and elapsed>10):
-                    log.warn("Warning: cannot move cursor")
-                    log.warn(" (%i events)", count+1)
-                    self.cursor_errors = [now, 1]
-                else:
-                    self.cursor_errors[1] = count+1
-
+            if SetPhysicalCursorPos(x, y):
+                return True
+            #rate limit the warnings:
+            start, count = self.cursor_errors
+            now = monotonic()
+            elapsed = now-start
+            if count==0 or (count>1 and elapsed>10):
+                log.warn("Warning: cannot move cursor")
+                log.warn(" (%i events)", count+1)
+                self.cursor_errors = [now, 1]
+            else:
+                self.cursor_errors[1] = count+1
         except Exception as e:
             log("SetPhysicalCursorPos%s failed", pointer, exc_info=True)
             log.error("Error: failed to move the cursor:")
             log.error(" %s", e)
-        return pointer
+        return False
 
     def clear_keys_pressed(self):
         keystate = (BYTE*256)()
