@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,7 +8,9 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000041.html
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
+import shlex
 import signal
+from time import monotonic
 from subprocess import Popen, PIPE, call
 import os.path
 
@@ -17,7 +19,7 @@ from xpra.util import envbool, envint
 from xpra.os_util import (
     shellsub,
     setuidgid, getuid, getgid,
-    strtobytes, bytestostr, osexpand, monotonic_time,
+    strtobytes, bytestostr, osexpand,
     pollwait,
     POSIX, OSX,
     )
@@ -70,21 +72,13 @@ def osclose(fd):
 def create_xorg_device_configs(xorg_conf_dir, device_uuid, uid, gid):
     log = get_vfb_logger()
     log("create_xorg_device_configs(%s, %s, %i, %i)", xorg_conf_dir, device_uuid, uid, gid)
-    cleanups = []
     if not device_uuid:
-        return cleanups
+        return
 
     def makedir(dirname):
         log("makedir(%s)", dirname)
         os.mkdir(dirname)
         os.lchown(dirname, uid, gid)
-        def cleanup_dir():
-            try:
-                log("cleanup_dir() %s", dirname)
-                os.rmdir(dirname)
-            except Exception as e:
-                log("failed to cleanup %s: %s", dirname, e)
-        cleanups.insert(0, cleanup_dir)
 
     #create conf dir if needed:
     d = xorg_conf_dir
@@ -103,11 +97,6 @@ def create_xorg_device_configs(xorg_conf_dir, device_uuid, uid, gid):
         ) :
         f = save_input_conf(xorg_conf_dir, i, dev_type, device_uuid, uid, gid)
         conf_files.append(f)
-    def cleanup_input_conf_files():
-        for f in conf_files:
-            os.unlink(f)
-    cleanups.insert(0, cleanup_input_conf_files)
-    return cleanups
 
 #create individual device files:
 def save_input_conf(xorg_conf_dir, i, dev_type, device_uuid, uid, gid):
@@ -162,7 +151,6 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
     if not xvfb_str:
         raise InitException("the 'xvfb' command is not defined")
 
-    cleanups = []
     log = get_vfb_logger()
     log("start_Xvfb%s", (xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, username, uinput_uuid))
     use_display_fd = display_name[0]=='S'
@@ -170,14 +158,17 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
     subs = {}
     def pathexpand(s):
         return osexpand(s, actual_username=username, uid=uid, gid=gid, subs=subs)
+    etc_prefix = os.environ.get("XPRA_INSTALL_PREFIX", "")
+    if etc_prefix.endswith("/usr"):
+        etc_prefix = etc_prefix[:-4]
     subs.update({
         "DISPLAY"       : display_name,
         "XPRA_LOG_DIR"  : pathexpand(os.environ.get("XPRA_LOG_DIR")),
+        "XORG_CONFIG_PREFIX" : os.environ.get("XORG_CONFIG_PREFIX", etc_prefix),
         })
 
     #identify logfile argument if it exists,
     #as we may have to rename it, or create the directory for it:
-    import shlex
     xvfb_cmd = shlex.split(xvfb_str)
     if not xvfb_cmd:
         raise InitException("cannot start Xvfb, the command definition is missing!")
@@ -245,7 +236,7 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
             #create uinput device definition files:
             #(we have to assume that Xorg is configured to use this path..)
             xorg_conf_dir = pathexpand(get_Xdummy_confdir())
-            cleanups = create_xorg_device_configs(xorg_conf_dir, uinput_uuid, uid, gid)
+            create_xorg_device_configs(xorg_conf_dir, uinput_uuid, uid, gid)
 
     xvfb_executable = xvfb_cmd[0]
     if (xvfb_executable.endswith("Xorg") or xvfb_executable.endswith("Xdummy")) and pixel_depth>0:
@@ -317,7 +308,7 @@ def start_Xvfb(xvfb_str, vfb_geom, pixel_depth, display_name, cwd, uid, gid, use
         raise
     log("xvfb process=%s", xvfb)
     log("display_name=%s", display_name)
-    return xvfb, display_name, cleanups
+    return xvfb, display_name
 
 
 def kill_xvfb(xvfb_pid):
@@ -369,11 +360,11 @@ def xauth_add(filename, display_name, xauth_data, uid, gid):
             if getuid()==0 and uid:
                 setuidgid(uid, gid)
         xauth_cmd = ["xauth"]+xauth_args
-        start = monotonic_time()
+        start = monotonic()
         log = get_vfb_logger()
         log("xauth command: %s", xauth_cmd)
         code = call(xauth_cmd, preexec_fn=preexec)
-        end = monotonic_time()
+        end = monotonic()
         if code!=0 and (end-start>=10):
             log.warn("Warning: xauth command took %i seconds and failed" % (end-start))
             #took more than 10 seconds to fail, check for stale locks:
@@ -410,7 +401,7 @@ def check_xvfb_process(xvfb=None, cmd="Xvfb", timeout=0, command=None):
     return False
 
 def verify_display_ready(xvfb, display_name, shadowing_check=True, log_errors=True, timeout=VFB_WAIT):
-    from xpra.x11.bindings.wait_for_x_server import wait_for_x_server        #@UnresolvedImport
+    from xpra.x11.bindings.wait_for_x_server import wait_for_x_server        #@UnresolvedImport pylint: disable=import-outside-toplevel
     # Whether we spawned our server or not, it is now running -- or at least
     # starting.  First wait for it to start up:
     try:
